<?php

namespace App\Http\Controllers;

use App\Models\MatchModel;
use App\Models\PartnerPreference;
use App\Models\ProfileMaster;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Laravel\Sanctum\PersonalAccessToken;

class MatchController extends Controller
{
    public function findMatches(Request $request)
    {
        // Get authenticated user
        $user = $this->getAuthenticatedUser($request);
        if (!$user) {
            return response()->json(['status' => false, 'message' => 'User not authenticated'], 401);
        }

        $userId = $user->U_Id;

        // Fetch user preferences
        $preferences = PartnerPreference::where('U_Id', $userId)->first();
        if (!$preferences) {
            return response()->json(['status' => false, 'message' => 'No preferences set'], 400);
        }

        // Extract user preferences
        list($ageMin, $ageMax) = explode('-', $preferences->preferred_age_range ?? "18-50");
        list($heightMin, $heightMax) = explode('-', $preferences->preferred_height_range ?? "4.5-7.0");

        $incomeMin = $preferences->income_min ?? 0;
        $incomeMax = $preferences->income_max ?? PHP_INT_MAX;

        // ✅ Find potential matches based on user's preferences
        $matchesQuery = ProfileMaster::whereNotNull('dob')
            ->whereRaw("TIMESTAMPDIFF(YEAR, dob, CURDATE()) BETWEEN ? AND ?", [(int) $ageMin, (int) $ageMax])
            ->whereBetween('height', [(float) $heightMin, (float) $heightMax])
            ->whereBetween('income_min', [$incomeMin, $incomeMax])
            ->whereBetween('income_max', [$incomeMin, $incomeMax])
            ->where(function ($query) use ($preferences) {
                $query->where('religion', $preferences->preferred_religion)
                    ->orWhereNull('religion');
            })
            ->where(function ($query) use ($preferences) {
                $query->where('caste', $preferences->preferred_caste)
                    ->orWhereNull('caste');
            })
            ->where(function ($query) use ($preferences) {
                $query->where('education', $preferences->preferred_education)
                    ->orWhereNull('education');
            })
            ->where(function ($query) use ($preferences) {
                $query->where('city', $preferences->preferred_location)
                    ->orWhereNull('city');
            })
            ->where('U_Id', '!=', $userId) // Exclude self
            ->get();

        // ✅ Debugging: Check if any matches exist
        if ($matchesQuery->isEmpty()) {
            return response()->json(['status' => false, 'message' => 'No matches found']);
        }

        return response()->json([
            'message' => 'Matches Found',
            'matches' => $matchesQuery
        ]);
    }





    private function getAuthenticatedUser(Request $request)
    {
        $token = $request->bearerToken();
        if (!$token) {
            return null;
        }

        $personalAccessToken = PersonalAccessToken::findToken($token);
        if (!$personalAccessToken) {
            return null;
        }

        return $personalAccessToken->tokenable; // Returns the authenticated user
    }
}
