<?php

namespace App\Http\Controllers;

use App\Models\PartnerPreference;
use Illuminate\Http\Request;
use Laravel\Sanctum\PersonalAccessToken;

class PartnerPreferenceController extends Controller
{
    public function getAuthenticatedUser(Request $request)
    {
        $token = $request->bearerToken();
        if (!$token) {
            return null;
        }

        $personalAccessToken = PersonalAccessToken::findToken($token);
        if (!$personalAccessToken) {
            return null;
        }

        return $personalAccessToken->tokenable; // This returns the associated user
    }

    // Create Partner Preferences
    public function store(Request $request)
    {
        $user = $this->getAuthenticatedUser($request);
        if (!$user) {
            return response()->json(['message' => 'Unauthorized'], 403);
        }

        // Check if user already has preferences
        $existingPreference = PartnerPreference::where('U_Id', $user->U_Id)->first();
        if ($existingPreference) {
            return response()->json(['message' => 'Preferences already exist. Use update endpoint instead.'], 400);
        }

        $request->validate([
            'preferred_age_range' => 'nullable|string',
            'preferred_height_range' => 'nullable|string',
            'preferred_religion' => 'nullable|string',
            'preferred_caste' => 'nullable|string',
            'preferred_education' => 'nullable|string',
            'income_min' => 'nullable|numeric',
            'income_max' => 'nullable|numeric',
            'preferred_location' => 'nullable|string',
        ]);

        $preference = PartnerPreference::create([
            'U_Id' => $user->U_Id,
            'preferred_age_range' => $request->preferred_age_range,
            'preferred_height_range' => $request->preferred_height_range,
            'preferred_religion' => $request->preferred_religion,
            'preferred_caste' => $request->preferred_caste,
            'preferred_education' => $request->preferred_education,
            'income_min' => $request->income_min,
            'income_max' => $request->income_max,
            'preferred_location' => $request->preferred_location,
        ]);

        return response()->json(['message' => 'Preferences created successfully', 'preferences' => $preference], 201);
    }

    public function update(Request $request)
    {
        $user = $this->getAuthenticatedUser($request);
        if (!$user) {
            return response()->json(['message' => 'Unauthorized'], 403);
        }

        $preference = PartnerPreference::where('U_Id', $user->U_Id)->first();
        if (!$preference) {
            return response()->json(['message' => 'No preferences found. Use create endpoint instead.'], 404);
        }

        $request->validate([
            'preferred_age_range' => 'nullable|string',
            'preferred_height_range' => 'nullable|string',
            'preferred_religion' => 'nullable|string',
            'preferred_caste' => 'nullable|string',
            'preferred_education' => 'nullable|string',
            'income_min' => 'nullable|numeric',
            'income_max' => 'nullable|numeric',
            'preferred_location' => 'nullable|string',
        ]);

        $preference->update($request->all());

        return response()->json(['message' => 'Preferences updated successfully', 'preferences' => $preference], 200);
    }


    // Get Partner Preferences of the User
    public function show(Request $request)
    {
        $user = $this->getAuthenticatedUser($request);
        if (!$user) {
            return response()->json(['message' => 'Unauthorized'], 403);
        }

        $preference = PartnerPreference::where('U_Id', $user->U_Id)->first();

        if (!$preference) {
            return response()->json(['message' => 'No preferences found'], 404);
        }

        return response()->json(['preferences' => $preference], 200);
    }

    // Delete Partner Preferences
    public function destroy(Request $request)
    {
        $user = $this->getAuthenticatedUser($request);
        if (!$user) {
            return response()->json(['message' => 'Unauthorized'], 403);
        }

        $preference = PartnerPreference::where('U_Id', $user->U_Id)->first();

        if (!$preference) {
            return response()->json(['message' => 'No preferences found'], 404);
        }

        $preference->delete();
        return response()->json(['message' => 'Preferences deleted successfully'], 200);
    }
}
