<?php

namespace App\Http\Controllers;

use App\Models\SubscriptionPlan;
use Illuminate\Container\Attributes\Auth;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Laravel\Sanctum\PersonalAccessToken;

class SubscriptionPlanController extends Controller
{
    // Get all subscription plans (Users & Admins)
    public function index()
    {
        return response()->json(['message' => 'Subscription Plans Fetched Successfully', 'Subscription Plans' => SubscriptionPlan::all()], 200);
    }

    // Create a new subscription plan (Only Super Admin)
    public function store(Request $request)
    {
        $user = $this->getAuthenticatedUser($request);
        if (!$user || $user->role->Role_Name !== 'Super Admin') {
            return response()->json(['message' => 'Unauthorized'], 403);
        }

        $validator = Validator::make($request->all(), [
            'name' => 'required|string|max:255',
            'price' => 'required|numeric|min:0',
            'duration' => 'required', // in days
            'description' => 'nullable|string',
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => 'Validation failed', 'errors' => $validator->errors()], 422);
        }
        $plan = SubscriptionPlan::create($request->all());
        return response()->json(['message' => 'Subscription plan created successfully', 'plan' => $plan], 201);
    }

    // Update a subscription plan (Only Super Admin)
    public function update(Request $request, $id)
    {
        $user = $this->getAuthenticatedUser($request);
        if (!$user || $user->role->Role_Name !== 'Super Admin') {
            return response()->json(['message' => 'Unauthorized'], 403);
        }

        $plan = SubscriptionPlan::findOrFail($id);
        // dd($plan);
        $validator = Validator::make($request->all(), [
            'name' => 'sometimes|string|max:255',
            'price' => 'sometimes|numeric|min:0',
            'duration' => 'sometimes',
            'description' => 'nullable|string',
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => 'Validation failed', 'errors' => $validator->errors()], 422);
        }

        $plan->name = $request->input('name');
        $plan->price = $request->input('price');
        $plan->duration = $request->input('duration');
        $plan->description = $request->input('description');
        $plan->save();
        return response()->json(['message' => 'Subscription plan updated successfully', 'plan' => $plan], 200);
    }

    // Delete a subscription plan (Only Super Admin)
    public function destroy(Request $request, $id)
    {
        $user = $this->getAuthenticatedUser($request);
        if (!$user || $user->role->Role_Name !== 'Super Admin') {
            return response()->json(['message' => 'Unauthorized'], 403);
        }

        SubscriptionPlan::findOrFail($id)->delete();
        return response()->json(['message' => 'Subscription plan deleted successfully']);
    }

    // Helper function to extract authenticated user from token
    private function getAuthenticatedUser(Request $request)
    {
        $token = $request->bearerToken();
        if (!$token) {
            return null;
        }

        $personalAccessToken = PersonalAccessToken::findToken($token);
        if (!$personalAccessToken) {
            return null;
        }

        return $personalAccessToken->tokenable; // This returns the associated user
    }
}
