<?php

namespace App\Http\Controllers;

use App\Models\UserMaster;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;

class UserMasterController extends Controller
{

    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'U_Name' => 'required|string|max:255',
            'U_Email' => 'required|email|unique:user_masters,U_Email',
            'U_Password' => 'required|string|min:6',
            'U_Gender' => 'nullable|string',
            'U_DOB' => 'nullable|date',
            'U_Image' => 'nullable|image|mimes:jpeg,png,jpg|max:2048',
            'Role_Id' => 'nullable|exists:role_masters,Role_Id',
            'U_mobile' => 'nullable|string|unique:user_masters,U_mobile',
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        // Ensure only one Super Admin exists
        if ($request->Role_Id == 1 && UserMaster::where('Role_Id', 1)->exists()) {
            return response()->json(['error' => 'Super Admin already exists'], 403);
        }

        $user = new UserMaster();
        $user->U_Name = $request->U_Name;
        $user->U_Email = $request->U_Email;
        $user->U_Password = Hash::make($request->U_Password);
        $user->U_Status = 'Active';
        $user->U_Gender = $request->U_Gender;
        $user->U_DOB = $request->U_DOB;
        $user->Role_Id = 2;
        $user->U_mobile = $request->U_mobile;
        $user->save();

        // Generate API token
        $token = $user->createToken('auth_token')->plainTextToken;

        return response()->json([
            'message' => 'User Created Successfully',
            'user' => collect($user)->except(['U_Id']),
            'token' => $token
        ], 200);
    }

    public function GetAllUsers()
    {
        $users = UserMaster::all();
        return response()->json(['message' => 'User Details Fetched Successfully', 'users' => $users], 200);
    }

    // Retrieve a Single User
    public function show($id)
    {
        $user = UserMaster::findOrFail($id);
        return response()->json(['message' => 'User Fetched Successfully', 'user' => $user], 200);
    }

    // Update User
    public function update(Request $request, $id)
    {
        $user = UserMaster::findOrFail($id);

        // Validate input (password not required in update)
        $validator = Validator::make($request->all(), [
            'U_Name' => 'sometimes|string|max:255',
            'U_Email' => 'sometimes|email|unique:users_master,U_Email,' . $id . ',U_Id',
            'U_Password' => 'nullable|string|min:6',
            'U_Status' => 'nullable|string',
            'U_Gender' => 'nullable|string',
            'U_DOB' => 'nullable|date',
            'U_Image' => 'nullable|image|mimes:jpeg,png,jpg|max:2048',
            'Role_Id' => 'nullable|exists:role_master,Role_Id',
            'U_mobile' => 'nullable|string|unique:users_master,U_mobile,' . $id . ',U_Id',
        ]);

        if ($validator->fails()) {
            return response()->json(['errors' => $validator->errors()], 422);
        }

        // Prevent updating Role_Id if user is Super Admin
        if ($user->Role_Id == 1 && $request->has('Role_Id')) {
            return response()->json(['error' => 'Super Admin role cannot be updated'], 403);
        }

        // Handle image upload if exists
        if ($request->hasFile('U_Image')) {
            // Delete old image if exists
            if (!empty($user->U_Image)) {
                $oldImagePath = public_path(str_replace(url('/'), '', $user->U_Image));
                if (file_exists($oldImagePath)) {
                    unlink($oldImagePath);
                }
            }

            // Upload new image
            $image = $request->file('U_Image');
            $imageName = time() . '.' . $image->getClientOriginalExtension();
            $folderName = 'uploads/users';
            $image->move(public_path($folderName), $imageName);
            $user->U_Image = url($folderName . '/' . $imageName);
        }

        // Update only fields that are present in the request, except Role_Id if user is Super Admin
        $updateData = $request->only([
            'U_Name',
            'U_Email',
            'U_Status',
            'U_Gender',
            'U_DOB',
            'U_mobile'
        ]);

        // If password is provided, hash it before saving
        if ($request->filled('U_Password')) {
            $updateData['U_Password'] = Hash::make($request->U_Password);
        }

        // If user is NOT Super Admin, allow Role_Id update
        if ($user->Role_Id != 1 && $request->filled('Role_Id')) {
            $updateData['Role_Id'] = $request->Role_Id;
        }

        // Update the user
        $user->update($updateData);

        return response()->json([
            'message' => 'User Updated Successfully',
            'user' => collect($user)->except(['U_Id'])
        ], 200);
    }



    // Delete User
    public function destroy($id)
    {
        $user = UserMaster::findOrFail($id);

        // Prevent deleting the Super Admin
        if ($user->Role_Id == 1) {
            return response()->json(['error' => 'Super Admin cannot be deleted'], 403);
        }

        if ($user->U_Image) {
            Storage::disk('public')->delete($user->U_Image);
        }

        $user->delete();

        return response()->json(['message' => 'User deleted successfully'], 200);
    }
}
